﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace NineBtnTest
{
  /// <summary>
  /// Note that only three 'using' statements are required to compile and run this code,
  /// once the NineBtnCtrl.DLL control library (containing 3 elements) has been added to
  /// your toolbox or a reference to it added using Solution Explorer.
  /// </summary>
  public partial class Form1 : Form
  {
    int ii,jj,bno; bool btn9PanelEnabled=true;
    Color BackC=Color.SlateGray,ChkC=Color.LightCyan,BordC=Color.Black,OvC=Color.LightBlue,DnC=Color.Lime;
    Color oldBackC,oldChkC,oldBordC,oldOvC,oldDnC;
    string AppPath="";
    /// <summary>
    /// This Form goes to a lot of expense to demonstrate the color effects
    /// which can be produced for FlatButtonAppearance elements when a button's
    /// style is FlatStyle.Flat, which is the default for the btn9Panel buttons.
    /// These characteristics can of course be set using the Visual Studio designer
    /// as soon as you drop one of the btn9Panel variants onto a form; although
    /// this Form allows you to experiment with the colors at runtime and see the
    /// effects produced by each combination firsthand.
    /// </summary>
    public Form1()
    {
      InitializeComponent();
      #region  INITIALIZE CLICK EVENT HANDLERS FOR THE THREE BUTTON MATRIXES  
      btn9Panel1.Size = btn9Panel1.PanelSize;
      btn9Panel1.Click +=new EventHandler(btn9Panel1_Click);
      btn9Panel1.Enable();
      btn9Panel1.RepeatDelay = 1;
      horizPanel1.Click +=new EventHandler(horizPanel1_Click);
      horizPanel1.Enable();
      horizPanel1.RepeatDelay = 1;
      vertPanel1.Click +=new EventHandler(vertPanel1_Click);
      vertPanel1.Enable();
      vertPanel1.RepeatDelay = 1;
      #endregion
      #region  INITIALIZE BUTTON COLORS TO THE DEFAULTS FOR btn9Panel1  
      BackC = btn9Panel1.BtnBackColor; ChkC = btn9Panel1.BtnChkdColor;
      BordC = btn9Panel1.BtnBordColor; OvC = btn9Panel1.BtnOverColor; DnC = btn9Panel1.BtnDownColor;
      oldBackC = BackC; oldChkC = ChkC; oldBordC = BordC; oldOvC = OvC; oldDnC = DnC;
      GetBackClrBtn.BackColor = BackC;
      GetChkClrBtn.BackColor = ChkC; GetBordClrBtn.BackColor = BordC;
      GetOvClrBtn.BackColor = OvC; GetDnClrBtn.BackColor = DnC;
      #endregion
      AppPath = Application.StartupPath;
    }
    ///  btn9Panel1_Click()  <summary>
    /// This is the Click() event tied to the btn9Panel 3x3 button matrix
    /// which is raised at every timer tick during a mouse down/up cycle.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void btn9Panel1_Click(object sender, EventArgs e)
    {
      ii = ii + btn9Panel1.Dx; jj = jj + btn9Panel1.Dy; bno = btn9Panel1.ClickButton;
      label1.Text = btn9Panel1.Name + "." +
        btn9Panel1.Btn9Arr[bno].Name+": " + ii.ToString() + " " + jj.ToString()+
        "\nDirection: "+btn9Panel1.Dirct[bno];
    }
    ///  horizPanel1_Click()  <summary>
    /// This is the Click() event tied to the horizPanel 1x2 button matrix
    /// which is raised at every timer tick during a mouse down/up cycle.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void horizPanel1_Click(object sender, EventArgs e)
    {
      ii = ii + horizPanel1.Dx; jj = jj + horizPanel1.Dy; bno = horizPanel1.ClickButton;
      label1.Text = horizPanel1.Name + "." +
        horizPanel1.Btn9Arr[bno].Name+": " + ii.ToString() + " " + jj.ToString()+
        "\nDirection: "+horizPanel1.Dirct[bno];
    }
    ///  vertPanel1_Click()  <summary>
    /// This is the Click() event tied to the vertPanel 2x1 button matrix
    /// which is raised at every timer tick during a mouse down/up cycle.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void vertPanel1_Click(object sender, EventArgs e)
    {
      ii = ii + vertPanel1.Dx; jj = jj + vertPanel1.Dy; bno = vertPanel1.ClickButton;
      label1.Text = vertPanel1.Name + "." +
        vertPanel1.Btn9Arr[bno].Name+": " + ii.ToString() + " " + jj.ToString()+
        "\nDirection: "+vertPanel1.Dirct[bno];
    }
    #region  COLOR MODIFICATION ROUTINES  
    public void ModifyColors(bool normal)
    {
      if (normal)
      {
        BackC = Color.SlateGray; ChkC=Color.LightCyan; BordC=Color.Black; OvC=Color.LightBlue; DnC=Color.Lime;
        GetChkClrBtn.BackColor = ChkC; GetBordClrBtn.BackColor = BordC;
        GetOvClrBtn.BackColor = OvC; GetDnClrBtn.BackColor = DnC;
        GetBackClrBtn.BackColor = BackC;
        oldBackC = BackC; oldChkC = ChkC; oldBordC = BordC; oldOvC = OvC; oldDnC = DnC;
      }
      btn9Panel1.BtnBackColor = BackC; btn9Panel1.BtnBordColor = BordC;
      btn9Panel1.BtnChkdColor = ChkC;  btn9Panel1.BtnDownColor = DnC;
      btn9Panel1.BtnOverColor = OvC;
    }
    ///  checkColors()  <summary>
    /// This either sets the color of the 3x3 button matrix buttons to the latest color 
    /// chosen using the Windows Color Dialog or to the default, for each of the five
    /// style elements (Background, Checked, Border, MouseOver, and MouseDown colors).
    /// </summary>
    public void checkColors()
    {
      if (BackChkBox.Checked) BackC = GetBackClrBtn.BackColor; else BackC = oldBackC;
      if (ChkChkBox.Checked) ChkC = GetChkClrBtn.BackColor; else ChkC = oldChkC;
      if (BordChkBox.Checked) BordC = GetBordClrBtn.BackColor; else BordC = oldBordC;
      if (OvChkBox.Checked) OvC = GetOvClrBtn.BackColor; else OvC = oldOvC;
      if (DnChkBox.Checked) DnC = GetDnClrBtn.BackColor; else DnC = oldDnC;
    }
    public void CheckModify() { checkColors(); ModifyColors(false); }
    public bool GetDlgColor() { return (colorDialog1.ShowDialog() == DialogResult.OK); }
    #endregion
    #region  COLOR-SETTING BUTTON CONTROL AND CHECKBOX EVENT HANDLERS  
    private void GetBackClrBtn_Click(object sender, EventArgs e)
    {
      if (GetDlgColor())
      {
        oldBackC = BackC; BackC = colorDialog1.Color; GetBackClrBtn.BackColor = BackC;
        BackChkBox.Checked = true; CheckModify();
      }
    }
    private void GetChkClrBtn_Click(object sender, EventArgs e)
    {
      if (GetDlgColor()) 
      {
        oldChkC = ChkC; ChkC = colorDialog1.Color; GetChkClrBtn.BackColor = ChkC;
        ChkChkBox.Checked = true; CheckModify();
      }
    }
    private void GetBordClrBtn_Click(object sender, EventArgs e)
    {
      if (GetDlgColor()) 
      {
        oldBordC = BordC; BordC = colorDialog1.Color; GetBordClrBtn.BackColor = BordC;
        BordChkBox.Checked = true; CheckModify();
      }
    }
    private void GetOvClrBtn_Click(object sender, EventArgs e)
    {
      if (GetDlgColor()) 
      {
        oldOvC = OvC; OvC = colorDialog1.Color; GetOvClrBtn.BackColor = OvC;
        OvChkBox.Checked = true; CheckModify();
      }
    }
    private void GetDnClrBtn_Click(object sender, EventArgs e)
    {
      if (GetDlgColor()) 
      {
        oldDnC = DnC; DnC = colorDialog1.Color; GetDnClrBtn.BackColor = DnC;
        DnChkBox.Checked = true; CheckModify();
      }
    }
    private void ResetClrBtn_Click(object sender, EventArgs e)
    { clearBtnImages(); if (!btn9PanelEnabled) btn9Panel1.Enable(); btn9PanelEnabled = true;
      ModifyColors(true); }
    private void ChkBox_Click(object sender, EventArgs e) { CheckModify(); }
    #endregion
    /// <summary>
    /// If you should like to have buttons more like the common GUI style,
    /// this runtime code calls clearBtnImages() and loadBtnImages() to
    /// support providing 16x16 bitmaps from eighteen .bmp files for the 
    /// btn9Panel button arrays.  There are vastly more efficient ways of
    /// storing the bitmap data in a single disk file or in an image list,
    /// but this code is written so that you can use your own bitmaps if
    /// you want with little modification.
    /// </summary>
    public void clearBtnImages()
    {
      int k;
      for (k=1; (k <= 9); k++)
      {
        if (btn9Panel1.Btn9Arr[k].BackgroundImage != null)
        {
          btn9Panel1.Btn9Arr[k].BackgroundImage.Dispose();
          btn9Panel1.Btn9Arr[k].BackgroundImage = null;
        }
      }
    }
    /// <summary>
    /// loadBtnImages() looks in the application startup path for a
    /// series of .bmp files with names ARRxD and ARRxE, where D is
    /// for disabled buttons and E for enabled buttons, and x=1..9.
    /// If it doesn't find one of these files, it clears all of the
    /// button background images, and the FlatStyle attributes of the
    /// buttons are then displayed (clicking the Reset button clears
    /// them as well).
    /// </summary>
    /// <param name="panelIsEnabled"></param>
    public void loadBtnImages(bool panelIsEnabled)
    {
      int k = 1; string s2;
      if (panelIsEnabled) s2 = "E"; else s2 = "D";
      while (k <= 9)
      {
        Bitmap bm; string s = k.ToString();
        try
        {
          bm = new Bitmap(AppPath+"\\ARR"+s+s2+".bmp");
          btn9Panel1.Btn9Arr[k].BackgroundImage = bm;
        }
        catch 
        {
          k = 11;
        }
        k++; 
      }
      if (k == 12) clearBtnImages();
    }
    /// <summary>
    /// The StdOnOffBtn click event toggles the 'standard' button
    /// appearance contributed by the background bitmaps between
    /// enabled and disabled states, also enabling and disabling the
    /// pan/scroll function of the buttons.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void StdOnOffBtn_Click(object sender, EventArgs e)
    {
      if (btn9PanelEnabled) { btn9Panel1.Disable(); btn9PanelEnabled = false; }
      else { btn9Panel1.Enable(); btn9PanelEnabled = true; }
      loadBtnImages(btn9PanelEnabled);
    }
  }
}
