﻿using System;
using System.Drawing;
using System.Windows.Forms;
//
// The following using statements are needed to compile this code:
//
// using System;
// using System.Drawing;
// using System.Windows.Forms;
//
namespace NineBtnCtrl
{
  ///  Btn9Button class <summary>     
  /// <para>
  /// Ten instances of the Btn9Button class are allocated by the Btn9Panel constructor.
  /// </para><para></para>■<para>
  /// The functions of eight of them, given numbers 1..8, are to respond individually to mouse down
  /// events by enabling a timer, and to disable it on a mouse up.  Button 9 is in the center, and
  /// the eight are arranged around it in numeric order by columns starting from the top left.
  /// </para><para></para>■<para>
  /// Buttons 1,3,6,8 are at the corners in the NW,SW,NE,SE directions.  Buttons 2,4,5,7 are at the
  /// edges in the W,N,S,E directions.  Scrolling or panning activity is then coordinated by setting
  /// two fields, Dx, and Dy, to +1 or -1 as the direction indicates; and for graphical purposes an
  /// additional field, GrDy, is set to -Dy because of the orientation of graphical screen Y-coordinates.
  /// </para><para></para>■<para>
  /// Clicking one of the eight buttons actually results in the Click event for the Btn9Panel being
  /// raised by the timer tick handler, so that control can be transferred to the application in which
  /// the panel has been installed by means of the panel's Click event handler; and all that is needed
  /// is that such a handler be created in the application as rptBtn9Panel1_Click(object sender,
  /// EventArgs e) and that the actual code to perform scrolling or panning of graphical objects (or to
  /// increment or decrement array indexes, etc.) be implemented in or called by that handler.
  /// </para><para></para>■<para>
  /// Button 0 is not used in this version of the class.  There are variants on the scheme featuring
  /// only two active buttons, placed horizontally or vertically, for single-direction scrolling.
  /// Button 9 isn't given any dedicated function, although its suggested use is as a mode toggle to
  /// switch between multiple functions of the button matrix or to activate or deactivate the scrolling
  /// functions of the other buttons.
  /// </para><para></para>■<para>
  /// The BtnID accessor provides access to the button number (1..8), and the panel class contains an
  /// array of strings giving the pan directions; i.e., "NE", "W", "SE", etc., for use in coordinating
  /// your scrolling code with other features as necessary.  When a button is pressed, the ClickButton
  /// field contains the BtnID value for that button (and when the mouse button is released it will be
  /// zero again).
  /// </para>
  /// </summary>
  public class Btn9Button : Button     
  {
    private int btnID = 0;
    public Btn9Button(int ID) { btnID = ID; }
    public void SetDimens(int x, int y, int wid, int ht)
    { base.Left = x; base.Top = y; base.Width = wid; base.Height = ht; }
    public int BtnID { get { return btnID; } }
  }
  ///  Btn9Panel class <summary>      
  /// An example showing how to use the Click() event for a Btn9Panel to hook the timer tick 
  /// generated by the Btn9Panel code is in Btn9Form.cs.<para>  </para> <para>
  /// <![CDATA[href="file://C:\CPRJ\NineBtnCtrl\NineBtnCtrl\Btn9Form.cs"]]>
  /// </para>
  /// At minimum, the form's InitializeComponent() statement should be followed by:
  /// <para>  </para><para>
  ///    btn9Panel1.Size = btn9Panel1.PanelSize;</para><para>
  ///    btn9Panel1.Click +=new EventHandler(btn9Panel1_Click);</para><para>
  ///    btn9Panel1.Enable();</para><para>
  ///    btn9Panel1.RepeatDelay = 10;</para><para>
  /// </para>  <para>
  /// and the scrolling code should be called by the Click() handler, defined as follows:
  /// </para>  <para>
  ///    private void btn9Panel1_Click(object sender, EventArgs e) </para><para> 
  /// </para><para></para>■<para>
  /// The form's default constructor creates a 60-pixel-square button matrix, which is large enough for most
  /// purposes; and an alternate constructor is provided allowing width and height to be specified.
  /// Both the Horiz and Vert fields are set to true in this class by the default constructor.
  /// Constructors which allow these fields and the dimensions to be set for the possible cases of
  /// a two-button matrix in column or row orientation are provided by the HorizPanel and VertPanel
  /// subclass constructors.  Compiling this file with a ClassLibrary target produces a .DLL file,
  /// NineBtnCtrl.dll, which can be added to a Visual Studio toolbox; and this provides three
  /// controls which can be dragged from the toolbox onto a form. These are the square 9-button matrix
  /// and the 2-button horizontal and vertical variants.  The border, background, mouseover, mousedown,
  /// and checked button colors can be configured in the VS designer; and modifying the PanelSize
  /// property in the designer results in recalculating the BtnRect[] measurements and adjusting
  /// the individual button positions and sizes accordingly.
  /// </para>
  /// </summary>
  public class Btn9Panel : Panel       
  {
    #region  Public Fields ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■  
    public Cursor[] RptCurs = { Cursors.Cross, Cursors.PanNW, Cursors.PanWest, Cursors.PanSW, Cursors.PanNorth,
      Cursors.PanSouth, Cursors.PanNE, Cursors.PanEast,  Cursors.PanSE, Cursors.Cross };
    public string[] Dirct = { "Out","NW","W","SW","N","S","NE","E","SE","In"};
    public Rectangle dimen = new Rectangle();
    int SqrSize = 60;
    ///  BtnRect[] <summary>
    /// <para>
    /// BtnRect[] contains the location and size of each of the nine Btn9Button controls in the
    /// Btn9Panel, and these dimensions can be accessed globally.  The Btn9PanelInit() method sets
    /// up these measurements, storing them in BtnRect[], and then transfers them to the individual
    /// Btn9Button instances.
    /// </para>
    /// </summary>
    public Rectangle[] BtnRect = new Rectangle[10];
    ///  ClickButton  <summary>
    /// ClickButton is zero except for during a mouse button press-release cycle.  During that time
    /// it contains the button number (1..9).
    /// </summary>
    public int ClickButton = 0;
    ///  Active[]  <summary>
    /// Active buttons are those added to the panel and for which the timer is enabled/disabled by
    /// a mouse button down/up cycle.  The values in Active[] are set by method Btn9PanelInit().
    /// </summary>
    public bool[] Active = new bool[10];  bool Valid = false, Horiz = true, Vert = true;
    ///  Btn9Arr[] <summary>
    /// <para>
    /// Btn9Arr[] contains the buttons instantiated by the panel constructor, whether they
    /// have been added as controls to the panel or not.  Button 0 is not added as a control to
    /// the panel, and can be used otherwise in any way you wish.  Button 9 is added to the panel,
    /// but is not active (the Active[9] array entry is false); and you can install an event
    /// handler for its Click event if you wish to use it for some purpose.  Note that since it
    /// is not active, the Enable() and Disable() methods do not add or remove mouse up/down
    /// event handlers on a button press as they do for the other eight buttons, and it also
    /// is not tied to or influenced by the Timer which is started and stopped by them.
    /// </para><para></para>■<para>
    /// The Btn9PanelInit() method initializes each button's location, size, style, name, and
    /// colors (the background, border, checked, mouseover, mousedown, and clicked colors).  
    /// If you want to use different colors, you can change them using SetNewBtnColor() or 
    /// the individual color accessors (which are also available via the VS designer).
    /// </para><para></para>■<para>
    /// Although the RptCurs[] array contains the direction-indicating cursors set by this
    /// initialization method, you can change that and/or create background bitmaps at the size
    /// you want for each button to indicate directionality and then programmatically set
    /// the background image for each button after the initialization code runs.
    /// </para><para></para>■<para>
    /// As is, the Btn9Panel class can be just dragged from a toolbox onto a form and
    /// instantiated using its default constructor, if you're satisfied with its look and
    /// performance.  You don't need to write much code at all to use it that as is, except
    /// for the code you need to scroll or pan your own items.  The example given above is
    /// sufficient to get it running, right out of the toolbox.
    /// </para>
    /// </summary>
    public Btn9Button[] Btn9Arr = new Btn9Button[10];
    /// <summary><para>
    /// You can set the default color scheme here before the VS designer sees it.  For
    /// example,</para><para> </para><para>
    ///   Btn9BackColor DarkSlateGray    515151    ButtonFace</para><para>
    ///   BtnBackColor  LightSlateGray   000000    ControlDark</para><para>
    ///   BtnBordColor  Silver           303030    ControlDarkDark</para><para>
    ///   BtnChkdColor  LightCyan        7A7A7A    LightCyan</para><para>
    ///   BtnDownColor  ControlDarkDark  7A7A7A    ControlLightLight</para><para>
    ///   BtnOverColor  DarkSlateGray    515151    Control</para><para>
    /// </para>
    /// </summary>
    Color btnBackColor = Color.SlateGray;
    Color btnChkdColor = Color.LightCyan;
    Color btnBordColor = Color.Black;
    Color btnOverColor = Color.LightBlue;
    Color btnDownColor = Color.Lime;
    Color btnActiveColor = Color.LightGray;
    Color btn9BackColor = Color.DarkSlateGray;
    #endregion
    #region  Panel Constructors / Parameters / Setup ■■■■■■■■■■■■■■■  
    ///  CODE CALLED BY ALL CONSTRUCTORS  <summary>
    /// Btn9PanelSetup() creates the timer and event handlers for the timer tick event and the
    /// mouse down/up events associated with the individual buttons, calculates the locations
    /// and sizes of the individual buttons, and then adds those buttons which are active for
    /// the variant being constructed (square, vertical, or horizontal) to the panel.
    /// </summary>
    /// <param name="width">Panel width in pixels</param>
    /// <param name="height">Panel height in pixels</param>
    private void Btn9PanelSetup(int width, int height)
    {
      try
      {
        for (int k=0; (k<=9); k++) { Btn9Arr[k] = new Btn9Button(k); }
        Btn9PanelInit(width, height);
        if (Valid) { SqrSize = width; if (height < SqrSize) SqrSize = height; }
        Rep9Timer = new Timer();
        Rep9Timer.Tick += new EventHandler(timerTick);
        RBMoUp = new MouseEventHandler(RepBtnN_MouseUp);
        RBMoDn = new MouseEventHandler(RepBtnN_MouseDown); PanelSize = new Size(width, height);
        for (int k=0; (k <= 9); k++)
        { if ((k > 0) & ((Active[k] | (k == 9)))) this.Controls.Add(Btn9Arr[k]); }
        SetInterval(10);
      }
      catch { Valid = false; SqrSize = 0; }
    }
    #region  CONSTRUCTORS / DESTRUCTOR  
    public Btn9Panel()
    { Horiz = true; Vert = true; SqrSize = 60; Btn9PanelSetup(SqrSize, SqrSize); }
    public Btn9Panel(bool hz, bool vt) 
    { dimen.X = 0; dimen.Y = 0; Horiz = hz; Vert = vt; dimen.Width = 20; dimen.Height = 20;
      if (hz) dimen.Width = 60; if (vt) dimen.Height = 60; Btn9PanelSetup(dimen.Width,dimen.Height); }
    public Btn9Panel(int wid, int ht)
    { dimen.X = 0; dimen.Y = 0; dimen.Width = wid; dimen.Height = ht; Horiz = true; Vert = true;
      Btn9PanelSetup(wid, ht); }
    public Btn9Panel(bool hz, bool vt, int wid, int ht)
    { dimen.X = 0; dimen.Y = 0; dimen.Width = wid; dimen.Height = ht; Horiz = hz; Vert = vt;
      Btn9PanelSetup(wid, ht); }
    ~Btn9Panel() { this.Done(); }
    void Done()
    {
      int k = 0, err = 0; Valid = false; Disable();
      while ((k <= 9) & (err == 0))
      {
        try { Btn9Arr[k].Cursor.Dispose(); Btn9Arr[k].Dispose(); Active[k] = false; }
        catch (Exception ex) { System.Windows.Forms.MessageBox.Show(ex.Message); err = -1; }
        k++;
      }
      if (err == 0) this.Dispose(false); 
    }
    #endregion
    public System.Drawing.Size PanelSize
    { get { System.Drawing.Size sz = new System.Drawing.Size(dimen.Width,dimen.Height); return sz; }
      set { dimen.Size = value; this.Size = value; Btn9PanelInit(dimen.Width,dimen.Height); }
    }
    #region  INITIALIZATION / REMEASUREMENT  
    public void Btn9PanelInit(int pWid, int pHt)
    {
      if ((Horiz | Vert) == false) { Horiz = true; Vert = true; }
      int Xdim,Ydim;
      Xdim = 1; Ydim = 1;
      if (Horiz) { if (Vert) { Xdim = 3;  } else { Xdim = 2; } }
      if (Vert) { if (Horiz) { Ydim = 3;  } else { Ydim = 2; } }
      int Wid = pWid / Xdim; int Ht = pHt / Ydim;
      #region  BUTTON MEASUREMENT  
      int i1,i2,j1,j2,k,k1,k2;
      for (k=1; (k <= 4); k++)
      {
          k1 = k; k2 = 9 - k; Active[k1] = false; Active[k2] = false;
          if (k1 > 3) i1 = 2; else i1 = 1; if (k1 > 3) j1 = 1; else j1 = k1; 
          if (k2 < 6) i2 = 2; else i2 = 3; if (k2 < 6) j2 = 3; else j2 = k2-5;
          if ((i1 == 1) & (Horiz))
          {
            if (j1 == 2) { Active[k1] = true; Active[k2] = true; if (!Vert) { j1 = 1; j2 = 1; i2 = 2; } }
            else if (Vert) { Active[k1] = true; Active[k2] = true; }
          }
          else if ((k == 4) & (Vert & (!Horiz)))
          {
            i1 = 1; i2 = 1; j1 = 1; j2 = 2; Active[k1] = true; Active[k2] = true;
          }
          if ((i1 == 2) & (Vert)) { Active[k1] = true; Active[k2] = true; }
          BtnRect[k1].X = (i1-1)*Wid; BtnRect[k1].Y = (j1-1)*Ht;
          BtnRect[k2].X = (i2-1)*Wid; BtnRect[k2].Y = (j2-1)*Ht;
          BtnRect[k1].Width = Wid; BtnRect[k1].Height = Ht;
          BtnRect[k2].Width = Wid; BtnRect[k2].Height = Ht;
      }
      BtnRect[9].X = Wid; BtnRect[9].Y = Ht; BtnRect[9].Width = Wid; BtnRect[9].Height = Ht;
      BtnRect[0].X = 0; BtnRect[0].Y = 0;
      if (Horiz) BtnRect[0].Width = BtnRect[7].X+Wid-BtnRect[2].X;
      else { BtnRect[0].Width = pWid; BtnRect[9].Width = 0; }
      if (Vert) BtnRect[0].Height = BtnRect[5].Y+Ht-BtnRect[4].Y;
      else { BtnRect[0].Height = pHt; BtnRect[9].Height = 0; }
      #endregion
      #region  BUTTON STYLING  
      Padding th0 = new Padding(0); Padding th1 = new Padding(1);
      for (k = 0; (k <= 9); k++)
      {
        Btn9Arr[k].Padding = th0;
        if ((Active[k]) | (k == 9))
        {
          Btn9Arr[k].Margin = th1;
          Btn9Arr[k].FlatStyle = FlatStyle.Flat;
          FlatButtonAppearance fba = Btn9Arr[k].FlatAppearance;
          fba.CheckedBackColor = btnChkdColor;// Color.LightCyan;
          fba.BorderColor = btnBordColor;// Color.Black;
          fba.BorderSize = 1;
          fba.MouseOverBackColor = btnOverColor;// Color.LightBlue;
          fba.MouseDownBackColor = btnDownColor;// Color.Lime;
          Btn9Arr[k].Cursor = RptCurs[k];
        }
        else
        {
          Btn9Arr[k].Margin = th0;
        }
        Btn9Arr[k].BackColor = btnBackColor;// Color.SlateGray;
        Btn9Arr[k].Name = "Btn9Arr["+k.ToString()+"]";
        Btn9Arr[k].Visible = (Active[k] | (k == 9));
      }
      #endregion
      for (k=0; (k <= 9); k++)
      {
        Btn9Arr[k].SetDimens(BtnRect[k].X, BtnRect[k].Y, BtnRect[k].Width, BtnRect[k].Height);
      }
      Valid = true; this.dimen.Width = Xdim*Wid; this.dimen.Height = Ydim*Ht;
    }
    #endregion
    #endregion
    #region  Unused ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■  
    /// <summary>
    /// If you're going to add code to support button 9 and mode switches for the other eight,
    /// these routines aren't in use and can set the eight to the same color or change one at
    /// a time to some other default of your choice.
    /// </summary>
    public void ClearButtonColors()
    {
      for (int k = 1; (k <= 9); k++)
      { if (Active[k]) Btn9Arr[k].BackColor = btnBackColor;/* Color.SlateGray; */}
      if (Active[9]) Btn9Arr[9].BackColor = btn9BackColor;// Color.DarkSlateGray;
    }
    public void SetButtonColor(int j)
    { if ((j>0) & (j <= 9)) if (Active[j]) Btn9Arr[j].BackColor = btnActiveColor;/* Color.LightGray;*/ }
    #endregion
    #region  Button Utility ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■  
    ///  SetNewButtonColor  <summary>
    /// If you're scrolling and reach a limit, you could change the button color to
    /// indicate that you can't scroll any further in that direction.
    /// </summary>
    /// <param name="j"></param>
    /// <param name="col"></param>
    public void SetNewButtonColor(int j, Color col)
    { if ((j>0) & (j <= 9)) Btn9Arr[j].BackColor = col; }
    #region SET COLORS FOR INDIVIDUAL BUTTONS HERE  
    public Color BtnChkdColor 
    { get { return btnChkdColor; } set { btnChkdColor = value; SetChkdColor(value); } }
    public Color BtnBordColor
    { get { return btnBordColor; } set { btnBordColor = value; SetBordColor(value); } }
    public Color BtnOverColor 
    { get { return btnOverColor; } set { btnOverColor = value; SetOverColor(value); } }
    public Color BtnDownColor 
    { get { return btnDownColor; } set { btnDownColor = value; SetDownColor(value); } }
    public Color BtnBackColor 
    { get { return btnBackColor; } set { btnBackColor = value; SetBackColor(value); } }
    public Color Btn9BackColor 
    { get { return btn9BackColor; } set { btn9BackColor = value; Btn9Arr[9].BackColor = value; } }
    #endregion
    #region SET A COLOR FOR ALL BUTTONS HERE  
    public void SetChkdColor(Color Check)
    { for (int k=1; (k <= 9); k++) Btn9Arr[k].FlatAppearance.CheckedBackColor = Check; }
    public void SetBordColor(Color Bord)
    { for (int k=1; (k <= 9); k++) Btn9Arr[k].FlatAppearance.BorderColor = Bord; } 
    public void SetOverColor(Color Over)
    { for (int k=1; (k <= 9); k++) Btn9Arr[k].FlatAppearance.MouseOverBackColor = Over; } 
    public void SetDownColor(Color Down)
    { for (int k=1; (k <= 9); k++) Btn9Arr[k].FlatAppearance.MouseDownBackColor = Down; }
    public void SetBackColor(Color Back) 
    { for (int k=1; (k <= 9); k++) Btn9Arr[k].BackColor = Back; }
    #endregion
    ///  dxdy[0..1,0..9] <summary>
    /// These are the increments associated with each of the buttons.  See the Dirct[] string array
    /// for the direction indicator tags.
    /// </summary>
    public int[,] dxdy = { { 0, 0 }, { -1, 1 }, { -1, 0 }, { -1, -1 }, { 0, 1 }, { 0, -1 }, { 1, 1 }, { 1, 0 }, { 1, -1 }, { 0, 0 } };
    public int Dx = 0; public int Dy = 0; public int GrDy = 0;
    ///  SetDxDy()  <summary>
    /// If you're using something other than a mouse down/up to start/stop scrolling, you can raise
    /// the up/down event for the button of your choice using Btn9Arr[].  If you need the Dx and Dy
    /// values set for some reason (if your code is written to require them to be set before a scroll
    /// increment is carried out), call SetDxDy() with the button number as parameter.  This lets you
    /// generate scroll calls (to your own code) without the use of the buttons or timer if you need to.
    /// </summary>
    /// <param name="k"></param>
    public void SetDxDy(int k)
    { if ((k > 0) & (k < 9)) { Dx = dxdy[k, 0]; Dy = dxdy[k, 1]; } else { Dx = 0; Dy = 0; } GrDy = -Dy; }
    ///  InBox() <summary>
    /// You can pass BtnRect[k] for k=0..9 to InBox() along with mouse coordinates relative to the
    /// panel origin to determine whether the mouse is over button k, in case you need to generate
    /// your own scroll event without a button down/up cycle involving that particular button or the timer.
    /// </summary>
    /// <param name="rect">A client region X Y Width Height structure</param>
    /// <param name="x">Mouse X-position in client coordinates</param>
    /// <param name="y">Mouse Y-position in client coordinates</param>
    /// <returns></returns>
    public bool InBox(System.Drawing.Rectangle rect, int x, int y)
    {
      bool b = false; 
      if ((x >= rect.X)&(y >= rect.Y)
        &(x <= (rect.X+rect.Width-1))
        &(y <= (rect.Y+rect.Height-1))) b = true;
      return b;
    }
    ///  GetBtnNo()  <summary>
    /// GetBtnNo() always returns the button ID number, duplicating the function of Btn9Button.BtnID(),
    ///  whereas SenderNo() returns the array index of the button in Btn9Arr[].
    /// </summary>
    /// <param name="rB"></param>
    /// <returns></returns>
    public int GetBtnNo(Btn9Button rB)
    { int res = 0; if (rB.GetType() == typeof(Btn9Button)) res = rB.BtnID; return res; }
    ///  SenderNo()  <summary>
    /// If you need to know the array index of one of the buttons, use SenderNo() to find it.
    /// </summary>
    /// <param name="sender"></param>
    /// <returns></returns>
    public int SenderNo(object sender)
    {
      int res = 0, k = 0, i = 1;
      while ((i <= 9) & (k == 0)) { if (sender.Equals(Btn9Arr[i])) k = i;  else i++; }
      if ((i <= 9) & (k != 0)) res = k;
      return res;
    }
    ///  RepBtnNo()  <summary>
    /// If you need a button number based on the position of the cursor over the panel
    /// independently of any mouse down/up events (if you've set up scrolling using the
    /// mouse hover or while you have disabled scrolling and the down/up handlers aren't
    /// hooked up), you can get the number and set Dx,Dy to the values corresponding to
    /// that button using function RepBtnNo().  Note that the button must be active.
    /// </summary>
    /// <param name="x"></param>
    /// <param name="y"></param>
    /// <returns></returns>
    public int RepBtnNo(int x, int y)
    {
      if (!Valid) return 0; int i=1,k = 0; bool fnd = false;
      while ((!fnd) & (i <= 9)) { if (Active[i]) fnd = InBox(BtnRect[i], x, y); if (fnd) k = i; else i++;  }
      if (fnd) SetDxDy(k);
      return k;
    }
    #endregion
    #region  Button Event Handlers ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■  
    ///  RepBtnN_MouseDown()  <summary><para>
    /// This single mouse down handler is used for all of the buttons on the panel.
    /// It enables and starts the timer, sets ClickButton to the ID of the button,
    /// and sets the scroll direction increments Dx and Dy.  The timer tick event
    /// handler then calls your Click() event handler until the timer is stopped.
    /// </para><para> </para><para>
    /// Your code only needs to ignore the event (do nothing in your event handler
    /// if BtnDown is false when the timer tick causes it to be called) in order to
    /// synchronize your scroller with the button down/up events.  If you're asynchronously
    /// polling both ClickButton and BtnDown in a tight loop and watching the timer,
    /// you might see events generated before BtnDown is true, and this could mean that
    /// Dx and Dy haven't been set yet.  If you're not polling this way, you wouldn't
    /// know that a Click() event occurred without the increments or button ID being set;
    /// so your best bet is to check BtnDown before doing anything to make sure you don't
    /// do anything until you know they've been set.  This code waits until the timer is
    /// started so that if your code gets a tick event without BtnDown=true it can make
    /// sure any other scrolling code which depends on ClickButton, Dx, or Dy can finish
    /// with what it's doing, provided you have arranged it so that ClickButton, Dx, or Dy
    /// being set in this code doesn't directly change the states of variables used in
    /// your scroll code while it is running.
    /// </para></summary>
    /// <param name="sender">One of the buttons in the button matrix</param>
    /// <param name="e">A mouse button press</param>
    private void RepBtnN_MouseDown(object sender, EventArgs e)
    { Rep9Timer.Interval=rep9Interval; Rep9Timer.Enabled=true; 
      Rep9Timer.Start(); ClickButton = SenderNo(sender); SetDxDy(ClickButton); BtnDown=true; }
    ///  RepBtnN_MouseUp()  <summary>
    /// This single mouse up handler is used for all of the buttons on the panel.
    /// It sets the scroll increments to zero and stops and disables the timer
    /// when you release the mouse button.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void RepBtnN_MouseUp(object sender, EventArgs e)
    { BtnDown=false; Rep9Timer.Stop(); Rep9Timer.Enabled=false; SetDxDy(0); ClickButton = 0; }
    #endregion
    #region  Timer Parameters ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■  
    private Timer Rep9Timer;
    ///  BtnDown  <summary>
    /// BtnDown is set true/false by the mouse down/up event handlers, and as long as it is true,
    /// the timer tick will transfer control to your Click() handler each time it ticks.  If you
    /// need to scroll only a short distance and then require a button release followed by another
    /// press in order to continue scrolling, you can set BtnDown to false to stop the scrolling
    /// (if your code will only start a scroll cycle if BtnDown=true) before the mouse button is 
    /// released.
    /// </summary>
    private bool BtnDown=false;
    private int rep9Interval=150;
    ///  MouseEventHandler references  <summary>
    /// These variables store local references to the mouse down/up handlers so that they
    /// can be installed and uninstalled without having to be re-allocated.
    /// </summary>
    private MouseEventHandler RBMoUp=null,RBMoDn=null;
    internal void timerTick(object o1, EventArgs e1)
    { if (BtnDown) {  base.OnClick(e1); } }
    protected override void OnClick(EventArgs e) { if (BtnDown) base.OnClick(e); }
    ///  Enable() / Disable()  <summary>
    /// Call Enable() and Disable() to turn on and turn off the active buttons in the
    /// button matrix without changing their active status.  These routines install
    /// and uninstall the mouse down/up event handlers for the buttons.  Note that you
    /// can't use Disable() while the timer is enabled (i.e., while one of the buttons
    /// in the button matrix is being 'pressed').
    /// </summary>
    public void Enable()
    { for (int i=1; i <= 9; i++)
      { if (Active[i]) { Btn9Arr[i].MouseUp += RBMoUp; Btn9Arr[i].MouseDown += RBMoDn; } } }
    public void Disable()
    { if (Rep9Timer.Enabled) return; for (int i=1; i <= 9; i++)
      { if (Active[i]) { Btn9Arr[i].MouseUp -= RBMoUp; Btn9Arr[i].MouseDown -= RBMoDn; } } }
    public void SetInterval(int v) { RepeatDelay = v; }
    public int RepeatDelay
    { get { return rep9Interval; } set { rep9Interval = value; if (rep9Interval<1) rep9Interval=1; } }
    #endregion
  }
  ///  HorizPanel class  <summary>    
  /// This provides a horizontal two-button pan/scroll control
  /// </summary>
  public class HorizPanel : Btn9Panel  
  {
    public HorizPanel() : base(true,false,60,20)
    {
    }
    public HorizPanel(int wid, int ht) : base(true,false,wid,ht)
    {
    }
  }
  ///  VertPanel class  <summary>     
  /// This provides a vertical two-button pan/scroll control
  /// </summary>
  public class VertPanel : Btn9Panel   
  {
    public VertPanel() : base(false, true, 20, 60)
    {
    }
    public VertPanel(int wid, int ht) : base(false, true, wid, ht)
    {
    }
  }
}   